
% lab04_ex_windows.m
% DtFT of windows, window importance in frequency analysis

clear all; close all;

M = 100;      % one-side number of window samples
N = 2*M+1;    % all samples
n = -M : M;   % sample indexes
text={'Rect','Triang','Hamm','Hann','Black','Kaiser','Cheb'};

% Window functions, i.e. "windows" in columns of matrix W
w(:,1) = boxcar(N);        w(:,5) = blackman(N);        
w(:,2) = bartlett(N);      w(:,6) = kaiser(N,10);
w(:,3) = hamming(N);       w(:,7) = chebwin(N,120);
w(:,4) = hanning(N);
[ N, Nw] = size(w);        % N window length, Nw number of windows

    figure
    plot(n,real(w)); xlabel('n'); title('Windows w(n)'); grid;
    axis([-(M+10) M+10 -0.1 1.1]); legend(text); pause

% Normalization required for correct interpretation of amplitude spectrum
w = w ./ repmat(sum(w),N,1);     % normalization (in column) to sum(w)=1 

% DtFT of windows
f = -1/10 : (1/N)/20 : 1/10;            % normalized freq ( f/fs )
for k = 1:length(f)
    bk(1:N) = exp( -j*2*pi*f(k)*n );    % reference Fourier harmonics
    W(k,1:Nw) = bk(1:N) * w(1:N,1:Nw);  % DtFT coeffs for all windows (in cols)
end
    figure
    subplot(211); plot(f,real(W)); title('Real(DtFT)'); grid;
    subplot(212); plot(f,imag(W)); title('Imag(DtFT)'); xlabel('f [Hz]'); grid; pause
    figure
    subplot(211); plot(f,abs(W)); title('Abs(DtFT)'); grid;
    subplot(212); plot(f,angle(W)); title('Angle(DtFT)'); xlabel('f [Hz]'); grid; pause
    figure
    subplot(111); plot(f,20*log10(abs(W))); xlabel('f [Hz]'); title('|W(f|');
    grid; axis([min(f) max(f) -160 20]);
    legend(text); pause

% Windowed signal
x = 2*cos(2*pi*1/20*n);     % signal
x = w .* repmat(x',1,Nw);   % its multiplication with many different windows
for k = 1:length(f)
    bk = exp( -j*2*pi*f(k)*n );
    X(k,:) = bk * x;
end
    figure
    subplot(111); plot(f,20*log10(abs(X))); xlabel('f [Hz]'); title('|X(f|');
    grid; axis([min(f) max(f) -160 20]);
    legend(text)

% Repeat the program for different values of N: 50, 100, 200, 500    